@@ -1,25 +1,11 @@
-# -*- coding: utf-8 -*-
-
 from xml.etree import cElementTree as etree
-# from xml.etree import ElementTree as etree
-import os
 import re
 import sys
 import time
+import pathlib
+import warnings
 
-VERSION = '1.29'
-
-CONSTRAINTS = {
-    'Time_Subdivision': 'Time subdivision of parent annotation\'s time interva'
-    'l, no time gaps allowed within this interval',
-    'Symbolic_Subdivision': 'Symbolic subdivision of a parent annotation. Anno'
-    'tations refering to the same parent are ordered',
-    'Symbolic_Association': '1-1 association with a parent annotation',
-    'Included_In': 'Time alignable annotations within the parent annotation\'s'
-    ' time interval, gaps are allowed'}
-
-MIMES = {'wav': 'audio/x-wav', 'mpg': 'video/mpeg', 'mpeg': 'video/mpg',
-         'xml': 'text/xml'}
+VERSION = '1.70'
 
 
 class Eaf:
@@ -27,7 +13,7 @@ class Eaf:
 
     .. note:: All times are in milliseconds and can't have decimals.
 
-    :var dict annotation_document: Annotation document TAG entries.
+    :var dict adocument: Annotation document TAG entries.
     :var list licenses: Licences included in the file of the form:
         ``(name, url)``.
     :var dict header: XML header.
@@ -72,20 +58,33 @@ class Eaf:
     :var dict annotations: Dictionary of annotations of the form:
         ``{id -> tier}``, this is only used internally.
     """
+    ETYPES = {'iso12620', 'ecv', 'cve_id', 'lexen_id', 'resource_url'}
+    CONSTRAINTS = {
+        'Time_Subdivision': "Time subdivision of parent annotation's time inte"
+        'rval, no time gaps allowed within this interval',
+        'Symbolic_Subdivision': 'Symbolic subdivision of a parent annotation. '
+        'Annotations refering to the same parent are ordered',
+        'Symbolic_Association': '1-1 association with a parent annotation',
+        'Included_In': 'Time alignable annotations within the parent annotatio'
+        "n's time interval, gaps are allowed"}
+    MIMES = {'wav': 'audio/x-wav', 'mpg': 'video/mpeg', 'mpeg': 'video/mpg',
+             'xml': 'text/xml'}
 
-    def __init__(self, file_path=None, author='pympi'):
+    def __init__(self, file_path=None, author='pympi',
+                 suppress_version_warning=False):
         """Construct either a new Eaf file or read on from a file/stream.
 
         :param str file_path: Path to read from, - for stdin. If ``None`` an
             empty Eaf file will be created.
         :param str author: Author of the file.
+        :param bool supress_version_warning: Suppress the warning for
+            unsupported EAF file versions
         """
-
         ctz = -time.altzone if time.localtime(time.time()).tm_isdst and\
             time.daylight else -time.timezone
-        self.maxts = None
-        self.maxaid = None
-        self.annotation_document = {
+        self.maxts = 1
+        self.maxaid = 1
+        self.adocument = {
             'AUTHOR': author,
             'DATE': time.strftime('%Y-%m-%dT%H:%M:%S{:0=+3d}:{:0=2d}').format(
                 ctz // 3600, ctz % 3600),
@@ -109,15 +108,14 @@ class Eaf:
         self.linked_file_descriptors = []
         self.media_descriptors = []
         self.properties = []
-        self.new_time, self.new_ann = 0, 0
 
         if file_path is None:
             self.add_linguistic_type('default-lt')
-            self.constraints = CONSTRAINTS.copy()
+            self.constraints = self.CONSTRAINTS.copy()
             self.properties.append(('lastUsedAnnotation', 0))
             self.add_tier('default')
         else:
-            parse_eaf(file_path, self)
+            parse_eaf(file_path, self, suppress_version_warning)
 
     def add_annotation(self, id_tier, start, end, value='', svg_ref=None):
         """Add an annotation.
@@ -134,6 +132,8 @@ class Eaf:
         """
         if self.tiers[id_tier][1]:
             raise ValueError('Tier already contains ref annotations...')
+        if not isinstance(start, int) or not isinstance(end, int):
+            raise ValueError('start and end must be an integer...')
         if start == end:
             raise ValueError('Annotation length is zero...')
         if start > end:
@@ -195,9 +195,8 @@ class Eaf:
         :param str value: Value of the external reference.
         :throws KeyError: if etype is not in the list of possible types.
         """
-        if etype not in\
-                ['iso12620', 'ecv', 'cve_id', 'lexen_id', 'resource_url']:
-            raise KeyError('etype not in possible types')
+        if etype not in self.ETYPES:
+            raise KeyError('etype not in {}'.format(self.ETYPES))
         self.external_refs[eid] = (etype, value)
 
     def add_language(self, lang_id, lang_def=None, lang_label=None):
@@ -285,7 +284,7 @@ class Eaf:
                           extension or an unknown mimetype.
         """
         if mimetype is None:
-            mimetype = MIMES[file_path.split('.')[-1]]
+            mimetype = self.MIMES[file_path.split('.')[-1]]
         self.media_descriptors.append({
             'MEDIA_URL': file_path, 'RELATIVE_MEDIA_URL': relpath,
             'MIME_TYPE': mimetype, 'TIME_ORIGIN': time_origin,
@@ -311,6 +310,10 @@ class Eaf:
     def add_ref_annotation(self, id_tier, tier2, time, value='',
                            prev=None, svg=None):
         """Add a reference annotation.
+        .. note:: When a timepoint matches two annotations the new reference
+        annotation will reference to the first annotation. To circumvent this
+        it's always safer to take the middle of the annotation you want to
+        reference to.
 
         :param str id_tier: Name of the tier.
         :param str tier2: Tier of the referenced annotation.
@@ -325,12 +328,23 @@ class Eaf:
         if self.tiers[id_tier][0]:
             raise ValueError('This tier already contains normal annotations.')
         ann = None
-        for aid, (begin, end, _, _) in self.tiers[tier2][0].items():
-            begin = self.timeslots[begin]
-            end = self.timeslots[end]
-            if begin <= time and end >= time:
-                ann = aid
-                break
+        # tier2 is an independent (aligned reference) tier
+        if self.tiers[tier2][0]:
+            for aid, (begin, end, _, _) in self.tiers[tier2][0].items():
+                begin = self.timeslots[begin]
+                end = self.timeslots[end]
+                if begin <= time and end >= time:
+                    ann = aid
+                    break
+        # tier2 is a reference tier, so we grab the top tier in the hierarchy
+        else:
+            for aid, (reference, _, _, _) in self.tiers[tier2][1].items():
+                refann = self.get_parent_aligned_annotation(aid)
+                begin = self.timeslots[refann[0]]
+                end = self.timeslots[refann[1]]
+                if begin <= time and end >= time:
+                    ann = aid
+                    break
         if not ann:
             raise ValueError('There is no annotation to reference to.')
         aid = self.generate_annotation_id()
@@ -352,7 +366,7 @@ class Eaf:
                           extension or an unknown mimetype.
         """
         if mimetype is None:
-            mimetype = MIMES[file_path.split('.')[-1]]
+            mimetype = self.MIMES[file_path.split('.')[-1]]
         self.linked_file_descriptors.append({
             'LINK_URL': file_path, 'RELATIVE_LINK_URL': relpath,
             'MIME_TYPE': mimetype, 'TIME_ORIGIN': time_origin,
@@ -401,15 +415,11 @@ class Eaf:
             self.tiers[tier_id] = ({}, {}, tier_dict, len(self.tiers))
 
     def child_tiers_for(self, id_tier):
-        """Give all child tiers for a tier.
+        """.. deprecated: 1.5
 
-        :param str id_tier: Name of the tier.
-        :returns: List of all children
-        :raises KeyError: If the tier is non existent.
+        Use :func:`get_child_tiers_for`
         """
-        self.tiers[id_tier]
-        return [m for m in self.tiers if 'PARENT_REF' in self.tiers[m][2] and
-                self.tiers[m][2]['PARENT_REF'] == id_tier]
+        return self.get_child_tiers_for(id_tier)
 
     def clean_time_slots(self):
         """Clean up all unused timeslots.
@@ -420,10 +430,8 @@ class Eaf:
         the flags for cleaning in the functions so that the cleaning is only
         performed afterwards.
         """
-        ts_in_tier = ((a[0], a[1]) for tier in self.tiers.values() for a in
-                      tier[0].values())
-        ts_in_tier = {a for b in ts_in_tier for a in b}
-        for a in ts_in_tier.symmetric_difference(self.timeslots):
+        ts = ((a[0], a[1]) for t in self.tiers.values() for a in t[0].values())
+        for a in {a for b in ts for a in b} ^ set(self.timeslots):
             del(self.timeslots[a])
 
     def copy_tier(self, eaf_obj, tier_name):
@@ -461,8 +469,8 @@ class Eaf:
             tier_name = '{}_{}_ftos'.format(tier1, tier2)
         self.add_tier(tier_name)
         ftos = []
-        ftogen = self.get_gaps_and_overlaps2(tier1, tier2, maxlen) if fast else\
-            self.get_gaps_and_overlaps(tier1, tier2, maxlen)
+        ftogen = self.get_gaps_and_overlaps2(tier1, tier2, maxlen) if fast\
+            else self.get_gaps_and_overlaps(tier1, tier2, maxlen)
         for fto in ftogen:
             ftos.append(fto)
             if fto[1]-fto[0] >= 1:
@@ -482,7 +490,7 @@ class Eaf:
         for t in eaf_out.get_tier_names():
             for ab, ae, value in eaf_out.get_annotation_data_for_tier(t):
                 if ab > end or ae < start:
-                    eaf_out.remove_annotation(t, (start-end)//2, False)
+                    eaf_out.remove_annotation(t, (ae - ab) // 2, False)
         eaf_out.clean_time_slots()
         return eaf_out
 
@@ -502,6 +510,7 @@ class Eaf:
             matches.
         :param bool safe: Ignore zero length annotations(when working with
             possible malformed data).
+        :returns: Name of the created tier.
         :raises KeyError: If the tier is non existent.
         """
         if tier_name is None:
@@ -515,16 +524,16 @@ class Eaf:
             if not safe or end > begin:
                 self.add_annotation(tier_name, begin, end, value)
         self.clean_time_slots()
+        return tier_name
 
     def generate_annotation_id(self):
         """Generate the next annotation id, this function is mainly used
         internally.
         """
-        if self.maxaid is None:
-            self.maxaid = 1
-            valid_anns = [a for a in self.annotations if re.match('.\d+', a)]
-            if valid_anns:
-                self.maxaid = max(int(x[1:]) for x in valid_anns)+1
+        if not self.maxaid:
+            valid_anns = [int(''.join(filter(str.isdigit, a)))
+                          for a in self.timeslots]
+            self.maxaid = max(valid_anns + [1])+1
         else:
             self.maxaid += 1
         return 'a{:d}'.format(self.maxaid)
@@ -538,11 +547,11 @@ class Eaf:
         """
         if time and time < 0:
             raise ValueError('Time is negative...')
-        if self.maxts is None:
-            self.maxts = 1
-            valid_ts = [a for a in self.timeslots if re.match('..\d+', a)]
-            if valid_ts:
-                self.maxts = max(int(x[2:]) for x in valid_ts)+1
+
+        if not self.maxts:
+            valid_ts = [int(''.join(filter(str.isdigit, a)))
+                        for a in self.timeslots]
+            self.maxts = max(valid_ts + [1])+1
         else:
             self.maxts += 1
         ts = 'ts{:d}'.format(self.maxts)
@@ -562,14 +571,52 @@ class Eaf:
         if self.tiers[id_tier][1]:
             return self.get_ref_annotation_at_time(id_tier, time)
         anns = self.tiers[id_tier][0]
-        return sorted(
-            [(self.timeslots[m[0]], self.timeslots[m[1]], m[2])
-                for m in anns.values() if
-                self.timeslots[m[0]] <= time and
-                self.timeslots[m[1]] >= time])
+        return sorted([(self.timeslots[m[0]], self.timeslots[m[1]], m[2])
+                       for m in anns.values() if
+                       self.timeslots[m[0]] <= time and
+                       self.timeslots[m[1]] >= time])
+
+    def get_annotation_data_after_time(self, id_tier, time):
+        """Give the annotation before a given time. When the tier contains
+        reference annotations this will be returned, check
+        :func:`get_ref_annotation_data_before_time` for the format. If an
+        annotation overlaps with ``time`` that annotation will be returned.
+
+        :param str id_tier: Name of the tier.
+        :param int time: Time to get the annotation before.
+        :raises KeyError: If the tier is non existent.
+        """
+        if self.tiers[id_tier][1]:
+            return self.get_ref_annotation_after_time(id_tier, time)
+        befores = self.get_annotation_data_between_times(
+            id_tier, time, self.get_full_time_interval()[1])
+        if befores:
+            return [min(befores, key=lambda x: x[0])]
+        else:
+            return []
+
+    def get_annotation_data_before_time(self, id_tier, time):
+        """Give the annotation before a given time. When the tier contains
+        reference annotations this will be returned, check
+        :func:`get_ref_annotation_data_before_time` for the format. If an
+        annotation overlaps with ``time`` that annotation will be returned.
+
+        :param str id_tier: Name of the tier.
+        :param int time: Time to get the annotation before.
+        :raises KeyError: If the tier is non existent.
+        """
+        if self.tiers[id_tier][1]:
+            return self.get_ref_annotation_before_time(id_tier, time)
+        befores = self.get_annotation_data_between_times(id_tier, 0, time)
+        if befores:
+            return [max(befores, key=lambda x: x[0])]
+        else:
+            return []
 
     def get_annotation_data_between_times(self, id_tier, start, end):
         """Gives the annotations within the times.
+        When the tier contains reference annotations this will be returned,
+        check :func:`get_ref_annotation_data_between_times` for the format.
 
         :param str id_tier: Name of the tier.
         :param int start: Start time of the annotation.
@@ -577,24 +624,38 @@ class Eaf:
         :returns: List of annotations within that time.
         :raises KeyError: If the tier is non existent.
         """
+        if self.tiers[id_tier][1]:
+            return self.get_ref_annotation_data_between_times(
+                id_tier, start, end)
         anns = ((self.timeslots[a[0]], self.timeslots[a[1]], a[2])
                 for a in self.tiers[id_tier][0].values())
         return sorted(a for a in anns if a[1] >= start and a[0] <= end)
 
     def get_annotation_data_for_tier(self, id_tier):
         """Gives a list of annotations of the form: ``(begin, end, value)``
-        When th tier contains reference annotations this will be returned,
+        When the tier contains reference annotations this will be returned,
         check :func:`get_ref_annotation_data_for_tier` for the format.
 
         :param str id_tier: Name of the tier.
         :raises KeyError: If the tier is non existent.
         """
         if self.tiers[id_tier][1]:
-            return self.get_ref_annotation_data_for_tier(id_tier, time)
+            return self.get_ref_annotation_data_for_tier(id_tier)
         a = self.tiers[id_tier][0]
         return [(self.timeslots[a[b][0]], self.timeslots[a[b][1]], a[b][2])
                 for b in a]
 
+    def get_child_tiers_for(self, id_tier):
+        """Give all child tiers for a tier.
+
+        :param str id_tier: Name of the tier.
+        :returns: List of all children
+        :raises KeyError: If the tier is non existent.
+        """
+        self.tiers[id_tier]
+        return [m for m in self.tiers if 'PARENT_REF' in self.tiers[m][2] and
+                self.tiers[m][2]['PARENT_REF'] == id_tier]
+
     def get_full_time_interval(self):
         """Give the full time interval of the file. Note that the real interval
         can be longer because the sound file attached can be longer.
@@ -602,8 +663,8 @@ class Eaf:
         :returns: Tuple of the form: ``(min_time, max_time)``.
         """
         return (0, 0) if not self.timeslots else\
-            (min(self.timeslots.values()),
-             max(self.timeslots.values()))
+            (min(v for v in self.timeslots.values() if v is not None),
+             max(v for v in self.timeslots.values() if v is not None))
 
     def get_gaps_and_overlaps(self, tier1, tier2, maxlen=-1):
         """Give gaps and overlaps. The return types are shown in the table
@@ -654,8 +715,10 @@ class Eaf:
         spkr2anns = sorted((self.timeslots[a[0]], self.timeslots[a[1]])
                            for a in self.tiers[tier2][0].values())
         line1 = []
-        isin = lambda x, lst: False if\
-            len([i for i in lst if i[0] <= x and i[1] >= x]) == 0 else True
+
+        def isin(x, lst):
+            return False if\
+                len([i for i in lst if i[0] <= x and i[1] >= x]) == 0 else True
         minmax = (min(spkr1anns[0][0], spkr2anns[0][0]),
                   max(spkr1anns[-1][1], spkr2anns[-1][1]))
         last = (1, minmax[0])
@@ -719,7 +782,9 @@ class Eaf:
                      self.get_annotation_data_for_tier(t)), reverse=True)
         if ad:
             last = (lambda x: (x[0][0], x[0][1], x[1]))(ad.pop())
-            thr = lambda x, y: maxlen == -1 or abs(x-y) < maxlen
+
+            def thr(x, y):
+                return maxlen == -1 or abs(x-y) < maxlen
             while ad:
                 (begin, end, _), current = ad.pop()
                 if last[2] == current and thr(begin, last[1]):
@@ -814,7 +879,7 @@ class Eaf:
         return self.linguistic_types[lingtype]
 
     def get_parameters_for_tier(self, id_tier):
-        """Give the parameter dictionary, this is usaable in :func:`add_tier`.
+        """Give the parameter dictionary, this is useable in :func:`add_tier`.
 
         :param str id_tier: Name of the tier.
         :returns: Dictionary of parameters.
@@ -844,6 +909,56 @@ class Eaf:
                 bucket.append((begin, end, value, rvalue))
         return bucket
 
+    def get_ref_annotation_data_after_time(self, id_tier, time):
+        """Give the ref annotation after a time. If an annotation overlaps
+        with `ktime`` that annotation will be returned.
+
+        :param str id_tier: Name of the tier.
+        :param int time: Time to get the annotation after.
+        :returns: Annotation after that time in a list
+        :raises KeyError: If the tier is non existent.
+        """
+        befores = self.get_ref_annotation_data_between_times(
+            id_tier, time, self.get_full_time_interval())
+        if befores:
+            return [min(befores, key=lambda x: x[0])]
+        else:
+            return []
+
+    def get_ref_annotation_data_before_time(self, id_tier, time):
+        """Give the ref annotation before a time. If an annotation overlaps
+        with ``time`` that annotation will be returned.
+
+        :param str id_tier: Name of the tier.
+        :param int time: Time to get the annotation before.
+        :returns: Annotation before that time in a list
+        :raises KeyError: If the tier is non existent.
+        """
+        befores = self.get_ref_annotation_data_between_times(id_tier, 0, time)
+        if befores:
+            return [max(befores, key=lambda x: x[0])]
+        else:
+            return []
+
+    def get_ref_annotation_data_between_times(self, id_tier, start, end):
+        """Give the ref annotations between times of the form
+        ``[(start, end, value, refvalue)]``
+
+        :param str tier: Name of the tier.
+        :param int start: End time of the annotation of the parent.
+        :param int end: Start time of the annotation of the parent.
+        :returns: List of annotations at that time.
+        :raises KeyError: If the tier is non existent.
+        """
+        bucket = []
+        for aid, (ref, value, _, _) in self.tiers[id_tier][1].items():
+            begin, end, rvalue, _ = self.tiers[self.annotations[ref]][0][ref]
+            begin = self.timeslots[begin]
+            end = self.timeslots[end]
+            if begin <= end and end >= begin:
+                bucket.append((begin, end, value, rvalue))
+        return bucket
+
     def get_ref_annotation_data_for_tier(self, id_tier):
         """"Give a list of all reference annotations of the form:
         ``[(start, end, value, refvalue)]``
@@ -854,11 +969,25 @@ class Eaf:
         """
         bucket = []
         for aid, (ref, value, prev, _) in self.tiers[id_tier][1].items():
-            refann = self.tiers[self.annotations[ref]][0][ref]
+            refann = self.get_parent_aligned_annotation(ref)
             bucket.append((self.timeslots[refann[0]],
                            self.timeslots[refann[1]], value, refann[2]))
         return bucket
 
+    def get_parent_aligned_annotation(self, ref_id):
+        """" Give the aligment annotation that a reference annotation belongs to directly, or indirectly through other
+        reference annotations.
+        :param str ref_id: Id of a reference annotation.
+        :raises KeyError: If no annotation exists with the id or if it belongs to an alignment annotation.
+        :returns: The alignment annotation at the end of the reference chain.
+        """
+        parentTier = self.tiers[self.annotations[ref_id]]
+        while 'PARENT_REF' in parentTier[2] and parentTier[2]['PARENT_REF'] and len(parentTier[2]) > 0:
+            ref_id = parentTier[1][ref_id][0]
+            parentTier = self.tiers[self.annotations[ref_id]]
+
+        return parentTier[0][ref_id]
+
     def get_secondary_linked_files(self):
         """Give all linked files."""
         return self.linked_file_descriptors
@@ -910,10 +1039,11 @@ class Eaf:
         :param str sep: Separator for the merged annotations.
         :param bool safe: Ignore zero length annotations(when working with
             possible malformed data).
+        :returns: Name of the created tier.
         :raises KeyError: If a tier is non existent.
         """
         if tiernew is None:
-            tiernew = '{}_merged'.format('_'.join(tiers))
+            tiernew = u'{}_merged'.format('_'.join(tiers))
         self.add_tier(tiernew)
         aa = [(sys.maxsize, sys.maxsize, None)] + sorted((
             a for t in tiers for a in self.get_annotation_data_for_tier(t)),
@@ -931,6 +1061,7 @@ class Eaf:
                 if end > l[1]:
                     l[1] = end
                 l[2].append(value)
+        return tiernew
 
     def remove_all_annotations_from_tier(self, id_tier, clean=True):
         """remove all annotations from a tier
@@ -950,7 +1081,9 @@ class Eaf:
 
     def remove_annotation(self, id_tier, time, clean=True):
         """Remove an annotation in a tier, if you need speed the best thing is
-        to clean the timeslots after the last removal.
+        to clean the timeslots after the last removal. When the tier contains
+        reference annotations :func:`remove_ref_annotation` will be executed
+        instead.
 
         :param str id_tier: Name of the tier.
         :param int time: Timepoint within the annotation.
@@ -958,10 +1091,12 @@ class Eaf:
         :raises KeyError: If the tier is non existent.
         :returns: Number of removed annotations.
         """
+        if self.tiers[id_tier][1]:
+            return self.remove_ref_annotation(id_tier, time, clean)
         removed = 0
-        for b in [a for a in self.tiers[id_tier][0].items() if
+        for b in sorted([a for a in self.tiers[id_tier][0].items() if
                   self.timeslots[a[1][0]] <= time and
-                  self.timeslots[a[1][1]] >= time]:
+                  self.timeslots[a[1][1]] >= time], reverse=True):
             del(self.tiers[id_tier][0][b[0]])
             del(self.annotations[b[0]])
             removed += 1
@@ -1030,8 +1165,7 @@ class Eaf:
         :param str url: URL of the license.
         """
         for k, v in self.licenses[:]:
-            if (name is None or name == k) and\
-                    (url is None or url == v):
+            if (name is None or name == k) and (url is None or url == v):
                 del(self.licenses[self.licenses.index((k, v))])
 
     def remove_linguistic_type(self, ling_type):
@@ -1082,10 +1216,30 @@ class Eaf:
         :param str value: Value of the property.
         """
         for k, v in self.properties[:]:
-            if (key is None or key == k) and\
-                    (value is None or value == v):
+            if (key is None or key == k) and (value is None or value == v):
                 del(self.properties[self.properties.index((k, v))])
 
+    def remove_ref_annotation(self, id_tier, time):
+        """Remove a reference annotation.
+
+        :param str id_tier: Name of tier.
+        :param int time: Time of the referenced annotation
+        :raises KeyError: If the tier is non existent.
+        :returns: Number of removed annotations.
+        """
+        removed = 0
+        bucket = []
+        for aid, (ref, value, _, _) in self.tiers[id_tier][1].items():
+            begin, end, rvalue, _ = self.tiers[self.annotations[ref]][0][ref]
+            begin = self.timeslots[begin]
+            end = self.timeslots[end]
+            if begin <= time and end >= time:
+                removed += 1
+                bucket.append(aid)
+        for aid in bucket:
+            del(self.tiers[id_tier][1][aid])
+        return removed
+
     def remove_secondary_linked_files(self, file_path=None, relpath=None,
                                       mimetype=None, time_origin=None,
                                       assoc_with=None):
@@ -1144,7 +1298,7 @@ class Eaf:
         :param str id_to: Target name of the tier.
         :throws KeyError: If the tier doesnt' exist.
         """
-        childs = self.child_tiers_for(id_from)
+        childs = self.get_child_tiers_for(id_from)
         self.tiers[id_to] = self.tiers.pop(id_from)
         self.tiers[id_to][2]['TIER_ID'] = id_to
         for child in childs:
@@ -1225,7 +1379,7 @@ def eaf_from_chat(file_path, codec='ascii', extension='wav'):
     since the library is designed for it. All CHAT headers will be added as
     Properties in the object and the headers that have a similar field in an
     Eaf file will be added there too. The file description of chat files can be
-    found `here <http://childes.psy.cmu.edu/manuals/CHAT.pdf>`_.
+    found `here <https://talkbank.org/manuals/CHAT.pdf>`_.
 
     :param str file_path: The file path of the .cha file.
     :param str codec: The codec, if the @UTF8 header is present it will choose
@@ -1296,7 +1450,7 @@ def eaf_from_chat(file_path, codec='ascii', extension='wav'):
     return eafob
 
 
-def parse_eaf(file_path, eaf_obj):
+def parse_eaf(file_path, eaf_obj, suppress_version_warning=False):
     """Parse an EAF file
 
     :param str file_path: Path to read from, - for stdin.
@@ -1306,11 +1460,19 @@ def parse_eaf(file_path, eaf_obj):
     if file_path == '-':
         file_path = sys.stdin
     # Annotation document
-    tree_root = etree.parse(file_path).getroot()
-    eaf_obj.annotation_document.update(tree_root.attrib)
-    del(eaf_obj.annotation_document[
-        '{http://www.w3.org/2001/XMLSchema-instance}noNamespaceSchemaLocation'
-        ])
+    try:
+        # py3.5 compat: etree.parse does not support pathlib.Path objects in py3.5.
+        tree_root = etree.parse(str(file_path)).getroot()
+    except etree.ParseError:
+        raise Exception('Unable to parse eaf, can you open it in ELAN?')
+
+    if not suppress_version_warning and \
+            tree_root.attrib['VERSION'] not in ['3.0', '2.8', '2.7']:
+        warnings.warn('Parsing unknown version of ELAN spec... '
+                      'This could result in errors...')
+    eaf_obj.adocument.update(tree_root.attrib)
+    del(eaf_obj.adocument['{http://www.w3.org/2001/XMLSchema-instance}noNamesp'
+                          'aceSchemaLocation'])
     tier_number = 0
     for elem in tree_root:
         # Licence
@@ -1330,11 +1492,12 @@ def parse_eaf(file_path, eaf_obj):
         # Time order
         elif elem.tag == 'TIME_ORDER':
             for elem1 in elem:
-                if int(elem1.attrib['TIME_SLOT_ID'][2:]) > eaf_obj.new_time:
-                    eaf_obj.new_time = int(elem1.attrib['TIME_SLOT_ID'][2:])
+                tsid = elem1.attrib['TIME_SLOT_ID']
+                tsnum = int(''.join(filter(str.isdigit, tsid)))
+                if tsnum and tsnum > eaf_obj.maxts:
+                    eaf_obj.maxts = tsnum
                 ts = elem1.attrib.get('TIME_VALUE', None)
-                eaf_obj.timeslots[elem1.attrib['TIME_SLOT_ID']] =\
-                    ts if ts is None else int(ts)
+                eaf_obj.timeslots[tsid] = ts if ts is None else int(ts)
         # Tier
         elif elem.tag == 'TIER':
             tier_id = elem.attrib['TIER_ID']
@@ -1345,9 +1508,10 @@ def parse_eaf(file_path, eaf_obj):
                     for elem2 in elem1:
                         if elem2.tag == 'ALIGNABLE_ANNOTATION':
                             annot_id = elem2.attrib['ANNOTATION_ID']
-                            if re.match('a\d+', annot_id) and\
-                                    int(annot_id[1:]) > eaf_obj.new_ann:
-                                eaf_obj.new_ann = int(annot_id[1:])
+                            annot_num = int(''.join(
+                                filter(str.isdigit, annot_id)))
+                            if annot_num and annot_num > eaf_obj.maxaid:
+                                eaf_obj.maxaid = annot_num
                             annot_start = elem2.attrib['TIME_SLOT_REF1']
                             annot_end = elem2.attrib['TIME_SLOT_REF2']
                             svg_ref = elem2.attrib.get('SVG_REF', None)
@@ -1357,19 +1521,20 @@ def parse_eaf(file_path, eaf_obj):
                                                svg_ref)
                             eaf_obj.annotations[annot_id] = tier_id
                         elif elem2.tag == 'REF_ANNOTATION':
-                            annotRef = elem2.attrib['ANNOTATION_REF']
+                            annot_ref = elem2.attrib['ANNOTATION_REF']
                             previous = elem2.attrib.get('PREVIOUS_ANNOTATION',
                                                         None)
-                            annotId = elem2.attrib['ANNOTATION_ID']
-                            if re.match('a\d+', annot_id) and\
-                                    int(annot_id[1:]) > eaf_obj.new_ann:
-                                eaf_obj.new_ann = int(annot_id[1:])
+                            annot_id = elem2.attrib['ANNOTATION_ID']
+                            annot_num = int(''.join(
+                                filter(str.isdigit, annot_id)))
+                            if annot_num and annot_num > eaf_obj.maxaid:
+                                eaf_obj.maxaid = annot_num
                             svg_ref = elem2.attrib.get('SVG_REF', None)
-                            ref[annotId] = (annotRef,
-                                            '' if not list(elem2)[0].text else
-                                            list(elem2)[0].text,
-                                            previous, svg_ref)
-                            eaf_obj.annotations[annotId] = tier_id
+                            ref[annot_id] = (annot_ref,
+                                             '' if not list(elem2)[0].text else
+                                             list(elem2)[0].text,
+                                             previous, svg_ref)
+                            eaf_obj.annotations[annot_id] = tier_id
             eaf_obj.tiers[tier_id] = (align, ref, elem.attrib, tier_number)
             tier_number += 1
         # Linguistic type
@@ -1395,10 +1560,21 @@ def parse_eaf(file_path, eaf_obj):
             cv_id = elem.attrib['CV_ID']
             ext_ref = elem.attrib.get('EXT_REF', None)
             descriptions = []
+
+            if 'DESCRIPTION' in elem.attrib:
+                eaf_obj.languages['und'] = (
+                    'http://cdb.iso.org/lg/CDB-00130975-001',
+                    'undetermined (und)')
+                descriptions.append(('und', elem.attrib['DESCRIPTION']))
             entries = {}
             for elem1 in elem:
                 if elem1.tag == 'DESCRIPTION':
                     descriptions.append((elem1.attrib['LANG_REF'], elem1.text))
+                elif elem1.tag == 'CV_ENTRY':
+                    cve_value = (elem1.text, 'und',
+                                 elem1.get('DESCRIPTION', None))
+                    entries['cveid{}'.format(len(entries))] = \
+                        ([cve_value], elem1.attrib.get('EXT_REF', None))
                 elif elem1.tag == 'CV_ENTRY_ML':
                     cem_ext_ref = elem1.attrib.get('EXT_REF', None)
                     cve_id = elem1.attrib['CVE_ID']
@@ -1426,7 +1602,7 @@ def indent(el, level=0):
     :param ElementTree.Element el: Current element.
     :param int level: Current level.
     """
-    i = '\n' + level*'\t'
+    i = '\n' + level * '\t'
     if len(el):
         if not el.text or not el.text.strip():
             el.text = i+'\t'
@@ -1448,20 +1624,17 @@ def to_eaf(file_path, eaf_obj, pretty=True):
     :param pympi.Elan.Eaf eaf_obj: Object to write.
     :param bool pretty: Flag to set pretty printing.
     """
-    rm_none = lambda x:\
-        dict((k, str(v)) for k, v in x.items() if v is not None)
+    def rm_none(x):
+        return {k: v if isinstance(v, str) else str(v) for k, v in x.items()
+                if v is not None}
     # Annotation Document
-    ANNOTATION_DOCUMENT = etree.Element('ANNOTATION_DOCUMENT',
-                                        eaf_obj.annotation_document)
-
+    ADOCUMENT = etree.Element('ANNOTATION_DOCUMENT', eaf_obj.adocument)
     # Licence
     for m in eaf_obj.licenses:
-        n = etree.SubElement(ANNOTATION_DOCUMENT, 'LICENSE',
-                             {'LICENSE_URL': m[1]})
+        n = etree.SubElement(ADOCUMENT, 'LICENSE', {'LICENSE_URL': m[1]})
         n.text = m[0]
     # Header
-    HEADER = etree.SubElement(ANNOTATION_DOCUMENT, 'HEADER', eaf_obj.header)
-
+    HEADER = etree.SubElement(ADOCUMENT, 'HEADER', eaf_obj.header)
     # Media descriptiors
     for m in eaf_obj.media_descriptors:
         etree.SubElement(HEADER, 'MEDIA_DESCRIPTOR', rm_none(m))
@@ -1471,55 +1644,45 @@ def to_eaf(file_path, eaf_obj, pretty=True):
     # Properties
     for k, v in eaf_obj.properties:
         etree.SubElement(HEADER, 'PROPERTY', {'NAME': k}).text = str(v)
-
     # Time order
-    TIME_ORDER = etree.SubElement(ANNOTATION_DOCUMENT, 'TIME_ORDER')
-    for t in sorted(eaf_obj.timeslots.items(),
-                    key=lambda x: int(x[0][2:])):
+    TIME_ORDER = etree.SubElement(ADOCUMENT, 'TIME_ORDER')
+    for t in sorted(eaf_obj.timeslots.items(), key=lambda x: int(x[0][2:])):
         etree.SubElement(TIME_ORDER, 'TIME_SLOT', rm_none(
             {'TIME_SLOT_ID': t[0], 'TIME_VALUE': t[1]}))
-
     # Tiers
-    for t in eaf_obj.tiers.items():
-        tier = etree.SubElement(ANNOTATION_DOCUMENT, 'TIER', rm_none(t[1][2]))
+    for t in sorted(eaf_obj.tiers.items(), key=lambda x: x[1][3]):
+        tier = etree.SubElement(ADOCUMENT, 'TIER', rm_none(t[1][2]))
         for a in t[1][0].items():
             ann = etree.SubElement(tier, 'ANNOTATION')
             alan = etree.SubElement(ann, 'ALIGNABLE_ANNOTATION', rm_none(
                 {'ANNOTATION_ID': a[0], 'TIME_SLOT_REF1': a[1][0],
                  'TIME_SLOT_REF2': a[1][1], 'SVG_REF': a[1][3]}))
-            etree.SubElement(alan, 'ANNOTATION_VALUE').text =\
-                str(a[1][2])
+            etree.SubElement(alan, 'ANNOTATION_VALUE').text = a[1][2]
         for a in t[1][1].items():
             ann = etree.SubElement(tier, 'ANNOTATION')
             rean = etree.SubElement(ann, 'REF_ANNOTATION', rm_none(
                 {'ANNOTATION_ID': a[0], 'ANNOTATION_REF': a[1][0],
                  'PREVIOUS_ANNOTATION': a[1][2], 'SVG_REF': a[1][3]}))
-            etree.SubElement(rean, 'ANNOTATION_VALUE').text =\
-                str(a[1][1])
-
+            etree.SubElement(rean, 'ANNOTATION_VALUE').text = a[1][1]
     # Linguistic types
     for l in eaf_obj.linguistic_types.values():
-        etree.SubElement(ANNOTATION_DOCUMENT, 'LINGUISTIC_TYPE', rm_none(l))
-
+        etree.SubElement(ADOCUMENT, 'LINGUISTIC_TYPE', rm_none(l))
     # Locales
     for lc, (cc, vr) in eaf_obj.locales.items():
-        etree.SubElement(ANNOTATION_DOCUMENT, 'LOCALE', rm_none(
+        etree.SubElement(ADOCUMENT, 'LOCALE', rm_none(
             {'LANGUAGE_CODE': lc, 'COUNTRY_CODE': cc, 'VARIANT': vr}))
-
     # Languages
     for lid, (ldef, label) in eaf_obj.languages.items():
-        etree.SubElement(ANNOTATION_DOCUMENT, 'LANGUAGE', rm_none(
+        etree.SubElement(ADOCUMENT, 'LANGUAGE', rm_none(
             {'LANG_ID': lid, 'LANG_DEF': ldef, 'LANG_LABEL': label}))
-
     # Constraints
     for l in eaf_obj.constraints.items():
-        etree.SubElement(ANNOTATION_DOCUMENT, 'CONSTRAINT', rm_none(
+        etree.SubElement(ADOCUMENT, 'CONSTRAINT', rm_none(
             {'STEREOTYPE': l[0], 'DESCRIPTION': l[1]}))
-
     # Controlled vocabularies
     for cvid, (descriptions, cv_entries, ext_ref) in\
             eaf_obj.controlled_vocabularies.items():
-        cv = etree.SubElement(ANNOTATION_DOCUMENT, 'CONTROLLED_VOCABULARY',
+        cv = etree.SubElement(ADOCUMENT, 'CONTROLLED_VOCABULARY',
                               rm_none({'CV_ID': cvid, 'EXT_REF': ext_ref}))
         for lang_ref, description in descriptions:
             des = etree.SubElement(cv, 'DESCRIPTION', {'LANG_REF': lang_ref})
@@ -1532,21 +1695,24 @@ def to_eaf(file_path, eaf_obj, pretty=True):
                 val = etree.SubElement(cem, 'CVE_VALUE', rm_none({
                     'LANG_REF': lang_ref, 'DESCRIPTION': description}))
                 val.text = value
-
     # Lexicon refs
     for l in eaf_obj.lexicon_refs.values():
-        etree.SubElement(ANNOTATION_DOCUMENT, 'LEXICON_REF', rm_none(l))
-
+        etree.SubElement(ADOCUMENT, 'LEXICON_REF', rm_none(l))
     # Exteral refs
     for eid, (etype, value) in eaf_obj.external_refs.items():
-        etree.SubElement(ANNOTATION_DOCUMENT, 'EXTERNAL_REF', rm_none(
+        etree.SubElement(ADOCUMENT, 'EXTERNAL_REF', rm_none(
             {'EXT_REF_ID': eid, 'TYPE': etype, 'VALUE': value}))
 
     if pretty:
-        indent(ANNOTATION_DOCUMENT)
+        indent(ADOCUMENT)
     if file_path == '-':
-        file_path = sys.stdout
-    elif os.access(file_path, os.F_OK):
-        os.rename(file_path, '{}.bak'.format(file_path))
-    etree.ElementTree(ANNOTATION_DOCUMENT).write(
-        file_path, xml_declaration=True, encoding='UTF-8')
+        try:
+            sys.stdout.write(etree.tostring(ADOCUMENT, encoding='unicode'))
+        except LookupError:
+            sys.stdout.write(etree.tostring(ADOCUMENT, encoding='UTF-8'))
+    else:
+        file_path = pathlib.Path(file_path)
+        if file_path.exists():
+            file_path.rename(file_path.with_suffix('.bak'))
+        etree.ElementTree(ADOCUMENT).write(
+            str(file_path), xml_declaration=True, encoding='UTF-8')
